<#
.SYNOPSIS
Script for setting up SQL failover cluster

.DESCRIPTION
This script will configure failover clustering to support a SQL failover cluster.


#>
param
(	
	#sql service username
	[Parameter(Mandatory=$true)]
	[string]$sqlusername,
	
	#sql service password
	[Parameter(Mandatory=$true)]
	[string]$sqlpassword,
	
	#sql service sa group
	[Parameter(Mandatory=$false)]
	[string]$sqlsagroup = $sqlusername,
	
	#sql instance name
	[Parameter(Mandatory=$false)]
	[string]$sqlinstance = "MSSQLSERVER",
	
	#sql AG name
	[Parameter(Mandatory=$false)]
	[string]$sqlagname = "AG01",
	
	#sql AG listener name
	[Parameter(Mandatory=$false)]
	[string]$sqlaglistenername = "AG01",
	
	#sql AG listener ip address
	[Parameter(Mandatory=$false)]
	[ValidateScript({ $_ -match [ipaddress] $_})]
	[string]$sqlaglistenerIP = "0.0.0.0",
	
	#sql AG listener ip mask
	[Parameter(Mandatory=$false)]
	[ValidateScript({ $_ -match "((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}0"})]
	[string]$sqlaglistenerMask = "0.0.0.0",
	
	#first node in cluster switch
	[switch]$FirstNode,
	
	#debug mode
	[switch]$VerboseMode
);

function Log($out) { 
    $out = [System.DateTime]::Now.ToString("yyyy.MM.dd hh:mm:ss") + " ---- " + $out; 
    Write-Output "$out"; 
}

function Log-Verbose($out) {
	if ($VerboseMode) {
		Log $out
	}
}

Log-Verbose "Parameters";
Log-Verbose "SQL Username:`t`t$sqlusername";
Log-Verbose "SQL Password:`t`t$sqlpassword";
Log-Verbose "SQL SA Group:`t`t$sqlsagroup";
Log-Verbose "SQL Instance Name:`t$sqlinstance";
Log-Verbose "SQL AG Name:`t`t$sqlagname";
Log-Verbose "SQL AG Listener Name:`t$sqlaglistenername";
Log-Verbose "SQL AG Listener IP:`t$sqlaglistenerIP";
Log-Verbose "SQL AG Listener Mask:`t$sqlaglistenerMask";
Log-Verbose "First Node Flag:`t$FirstNode";

#SQL Installation
$cmd = "D:\Setup.exe";
$cmdargs = @();

Log "Configuring SQL command for instance $sqlinstance...";

$cmdargs = @(
	"/ConfigurationFile=.\Configuration\SQL-AlwaysOn.ini",
	"/IAcceptSQLServerLicenseTerms",
	"/AGTSVCACCOUNT=$sqlusername",
	"/AGTSVCPASSWORD=$sqlpassword",
	"/SQLSVCACCOUNT=$sqlusername",
	"/SQLSVCPASSWORD=$sqlpassword",
	"/SQLSYSADMINACCOUNTS=$sqlsagroup",
	"/INSTANCENAME=$sqlinstance",
	"/FTSVCACCOUNT=NT Service\MSSQLFDLauncher`$$sqlinstance"
	);

Log "Starting SQL install...";
Write-Output "";
Log-Verbose $("$cmd " + $($cmdargs -replace [RegEx]::Escape($sqlpassword),"[HIDDEN FROM LOG]"));
Write-Output "";

$Error.Clear();
$PreviousPref = $ErrorActionPreference;
$ErrorActionPreference = "SilentlyContinue";
&$cmd $cmdargs | Out-Null;
$ErrorActionPreference = $PreviousPref;
if($LASTEXITCODE) {
    Log "SQL installation failure:"
    Write-Output "";
    Log $Error;
    Write-Output "";
    Log "See `"$env:ProgramFiles\Microsoft SQL Server\110\Setup Bootstrap\Log\Summary.txt`" for details.";
    exit $LASTEXITCODE;
}
Log-Verbose "SQL install complete.";

#enable Windows Firewall Rules for SQL
Log "Configuring firewall...";
foreach($service in $(Get-Service MSSQL*,SQL*,MSOLAP*)) {
    Log-Verbose ("Adding firewall rule for service " + $service.Name);
    New-NetFirewallRule -DisplayName $service.DisplayName -Profile Domain -Service $service.Name | Out-Null;
}

$sqlFirewallRuleCmd={
	netsh advfirewall firewall add rule name="SQL HADR EndPoint" dir=in action=allow protocol=TCP localport=5022
	netsh firewall set service type=fileandprint mode=enable profile=all
	}

Invoke-Command -ScriptBlock $sqlFirewallRuleCmd;
	
Log-Verbose "Done configuring firewall.";

Log "Configuring SQL AlwaysOn Availability Groups..."

$computername = $env:computername;
$fqdn = $computername + "." + $env:userdnsdomain;
$instance = @{$true="DEFAULT";$false=$sqlinstance}[$sqlinstance -eq "MSSQLSERVER"];
$replicaname = @{$true="$computername";$false="$($computername)\$($sqlinstance)"}[$sqlinstance -eq "MSSQLSERVER"];

Log-Verbose "Computername:`t$computername";
Log-Verbose "FQDN:`t`t$fqdn";
Log-Verbose "Instance:`t$instance";
Log-Verbose "Replica Name:`t$replicaname";

Import-Module "C:\Program Files (x86)\Microsoft SQL Server\110\Tools\PowerShell\Modules\SqlPS";

Log-Verbose "Enabling Availability groups on $computername";
Enable-SqlAlwaysOn -Path "SQLSERVER:\SQL\$computername\$instance" -Force;

Log-Verbose "Creating Endpoint [Hadr_endpoint] for $computername on port 5022";
$endpoint = New-SqlHadrEndpoint Hadr_endpoint -Port 5022 -Path "SQLSERVER:\SQL\$computername\$instance";

Log-Verbose "Granting $sqlusername connect permission on Endpoint [Hadr_endpoint]";
$query = "
	USE [master]
    GO 
	
	CREATE LOGIN [$sqlusername] FROM WINDOWS
	GO
	
	GRANT CONNECT ON ENDPOINT::[Hadr_endpoint] TO [$sqlusername]
	GO
	"
Invoke-SqlCmd -Query $query;

Log-Verbose "Starting Endpoint";
Set-SqlHadrEndpoint -InputObject $endpoint -State "Started" | Out-Null;

if($FirstNode) {
	Log-Verbose "Creating AG named $sqlagname and replica on $computername";
	$replica = New-SqlAvailabilityReplica -Name $replicaname -EndpointUrl "TCP://$($fqdn):5022" -FailoverMode "Automatic" -AvailabilityMode "SynchronousCommit" -ConnectionModeInSecondaryRole "AllowReadIntentConnectionsOnly" -AsTemplate -Version 11;
	$ag = New-SqlAvailabilityGroup -Name $sqlagname -AvailabilityReplica ($replica) -Path "SQLSERVER:\SQL\$computername\$instance";

	Log-Verbose "Creating AG listener named $sqlaglistenername on $sqlaglisternerip";
	$listener = New-SqlAvailabilityGroupListener -Name $sqlaglistenername -StaticIp "$($sqlaglistenerip)/$($sqlaglistenermask)" -Path "SQLSERVER:\SQL\$computername\$instance\AvailabilityGroups\$($sqlagname)" -port 1433;
} else {
	Log-Verbose "Finding primary AG node";
	$agresource = (Get-ClusterResource | ? ResourceType -eq "SQL Server Availability Group" | ? Name -eq "$sqlagname");
	
	if (!$agresource) {
		Write-Error "Could not find primary AG node for group $sqlagname";
		exit -1;
	} else {
		$primarynode = $agresource.OwnerNode.Name;
	}
	
	Log-Verbose "Found pirmary AG node for group $($sqlagname): $primarynode";
	
	Log-Verbose "Adding replica on $computername";
	$replica = New-SqlAvailabilityReplica -Name $replicaname -EndpointUrl "TCP://$($fqdn):5022" -FailoverMode "Automatic" -AvailabilityMode "SynchronousCommit" -ConnectionModeInSecondaryRole "AllowReadIntentConnectionsOnly" -Path "SQLSERVER:\SQL\$primarynode\$instance\AvailabilityGroups\$($sqlagname)";
	
	Log-Verbose "Joining AG $sqlagname";
	$ag = Join-SqlAvailabilityGroup -Path "SQLSERVER:\SQL\$computername\$instance" -Name $sqlagname;
}

Log-Verbose "SQL AG configuration complete.";