<#
.SYNOPSIS
Script for setting up SQL failover cluster

.DESCRIPTION
This script will configure failover clustering to support a SQL failover cluster.


#>
param
(	
	#sql service username
	[Parameter(Mandatory=$true)]
	[string]$sqlusername,
	
	#sql service password
	[Parameter(Mandatory=$true)]
	[string]$sqlpassword,
	
	#sql cluster object name
	[Parameter(Mandatory=$true)]
	[string]$sqlclustername,
	
	#sql service sa group
	[Parameter(Mandatory=$false)]
	[string]$sqlsagroup = $sqlusername,
	
	#sql instance name
	[Parameter(Mandatory=$false)]
	[string]$sqlinstance = "MSSQLSERVER",
	
	#sql cluster object ip address
	[Parameter(Mandatory=$false)]
	[ValidateScript({ $_ -match [ipaddress] $_})]
	[string]$sqlclusterIP = "0.0.0.0",
	
	#sql cluster object ip mask
	[Parameter(Mandatory=$false)]
	[ValidateScript({ $_ -match "((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}0"})]
	[string]$sqlclusterMask = "0.0.0.0",
	
	#data disk drive letter
	[Parameter(Mandatory=$false)]
    [ValidateScript({ $_ -match "[A-Za-z]"})]
	[string]$datadiskletter = "",
	
	#first node in cluster switch
	[switch]$FirstNode,
	
	#debug mode
	[switch]$VerboseMode
);

function Log($out) { 
    $out = [System.DateTime]::Now.ToString("yyyy.MM.dd hh:mm:ss") + " ---- " + $out; 
    Write-Output "$out"; 
}

function Log-Verbose($out) {
	if ($VerboseMode) {
		Log $out
	}
}

Log-Verbose "Parameters";
Log-Verbose "Data Disk Letter:`t$datadiskletter";
Log-Verbose "SQL Username:`t`t$sqlusername";
Log-Verbose "SQL Password:`t`t$sqlpassword";
Log-Verbose "SQL SA Group:`t`t$sqlsagroup";
Log-Verbose "SQL Instance Name:`t$sqlinstance";
Log-Verbose "SQL Cluster Name:`t$sqlclustername";
Log-Verbose "SQL Cluster IP:`t$sqlclusterIP";
Log-Verbose "SQL Cluster Mask:`t$sqlclusterMask";
Log-Verbose "First Node Flag:`t$FirstNode";

#SQL Installation
Log "Determining network configuration...";

$clusternet = $(Get-ClusterNetwork)[0].Name;

Log-Verbose "Cluster Network Name: $clusternet";

if ($sqlclusterIP -ne "0.0.0.0" -and $sqlclusterMask -ne "0.0.0.0") {
	Log "Using Static IP $sqlclusterIP/$subnetmask";
    $netstring = "IPv4;$sqlclusterIP;$clusternet;$sqlclusterMask";
} else {
	Log "Using DHCP"
    $netstring = "IPv4;DHCP;$clusternet";
}

Log-Verbose "Network string: $netstring";

$cmd = "D:\Setup.exe";
$cmdargs = @();

if ($FirstNode) {
	if ($datadiskletter -eq "") {
		Log "Using next available cluster disk";
		$clusterdisks = Get-ClusterResource | ? OwnerGroup -eq "Available Storage";

		$drive = $clusterdisks[0].Name;
		
		Log-Verbose "Finding drive letter of $drive...";
		
		$resource = gwmi MSCluster_Resource -Namespace root/mscluster | ?{$_.Name -eq $drive};
		$disk = gwmi -Namespace root/mscluster -Query "Associators of {$resource} Where ResultClass=MSCluster_Disk";
		$partition = gwmi -Namespace root/mscluster -Query "Associators of {$disk} Where ResultClass=MSCluster_DiskPartition";
		
		$letter = $partition.Path;
	} else {
		$letter = "$($datadiskletter):";
		Log "Using $letter as cluster data disk";

		Log-Verbose "Finding drive from letter $letter";

		$partition = gwmi MSCluster_DiskPartition -Namespace root/mscluster | ? Path -eq "$letter";
		$disk = gwmi -Namespace root/mscluster -Query "Associators of {$partition} Where ResultClass=MSCluster_Disk";
		$resource = gwmi -Namespace root/mscluster -Query "Associators of {$disk} Where ResultClass=MSCluster_Resource";
		
		$drive = $resource.Name;
	}

	Log-Verbose "Found $drive as $letter";
	
	Log "Configuring SQL command for instance $sqlinstance...";
	
	$cmdargs = @(
		"/ConfigurationFile=.\Configuration\SQL-FailoverCluster-FirstNode.ini",
		"/IAcceptSQLServerLicenseTerms",
		"/AGTSVCACCOUNT=$sqlusername",
		"/AGTSVCPASSWORD=$sqlpassword",
		"/SQLSVCACCOUNT=$sqlusername",
		"/SQLSVCPASSWORD=$sqlpassword",
		"/SQLSYSADMINACCOUNTS=$sqlsagroup",
		"/FAILOVERCLUSTERNETWORKNAME=$sqlclustername",
		"/FAILOVERCLUSTERIPADDRESSES=$netstring",
		"/FAILOVERCLUSTERDISKS=$drive",
		"/INSTALLSQLDATADIR=$letter",
		"/INSTANCENAME=$sqlinstance",
		"/SQLUSERDBLOGDIR=$letter\MSSQL11.$sqlinstance\MSSQL\Data",
		"/SQLTEMPDBDIR=$letter\MSSQL11.$sqlinstance\MSSQL\Data",
		"/SQLTEMPDBLOGDIR=$letter\MSSQL11.$sqlinstance\MSSQL\Data",
		"/FAILOVERCLUSTERGROUP=SQL Server ($sqlinstance)",
		"/FTSVCACCOUNT=NT Service\MSSQLFDLauncher`$$sqlinstance"
		);
} else {
	Log "Configuring SQL command for instance $sqlinstance...";
	
	$cmdargs = @(
		"/ConfigurationFile=.\Configuration\SQL-FailoverCluster-AdditionalNode.ini",
		"/IAcceptSQLServerLicenseTerms",
		"/AGTSVCACCOUNT=$sqlusername",
		"/AGTSVCPASSWORD=$sqlpassword",
		"/SQLSVCACCOUNT=$sqlusername",
		"/SQLSVCPASSWORD=$sqlpassword",
		"/FAILOVERCLUSTERNETWORKNAME=$sqlclustername",
		"/FAILOVERCLUSTERIPADDRESSES=$netstring",
		"/INSTANCENAME=$sqlinstance",
		"/FAILOVERCLUSTERGROUP=SQL Server ($sqlinstance)",
		"/FTSVCACCOUNT=NT Service\MSSQLFDLauncher`$$sqlinstance"
		);
}

Log "Starting SQL install...";
Write-Output "";
Log-Verbose $("$cmd " + $($cmdargs -replace [RegEx]::Escape($sqlpassword),"[HIDDEN FROM LOG]"));
Write-Output "";

$Error.Clear();
$PreviousPref = $ErrorActionPreference;
$ErrorActionPreference = "SilentlyContinue";
&$cmd $cmdargs | Out-Null;
$ErrorActionPreference = $PreviousPref;
if($LASTEXITCODE) {
    Log "SQL installation failure:"
    Write-Output "";
    Log $Error;
    Write-Output "";
    Log "See `"$env:ProgramFiles\Microsoft SQL Server\110\Setup Bootstrap\Log\Summary.txt`" for details.";
    exit $LASTEXITCODE;
}
Log-Verbose "SQL install complete.";

Log "Configuring firewall...";
foreach($service in $(Get-Service MSSQL*,SQL*,MSOLAP*)) {
    Log-Verbose ("Adding firewall rule for service " + $service.Name);
    New-NetFirewallRule -DisplayName $service.DisplayName -Profile Domain -Service $service.Name | Out-Null;
}
Log-Verbose "Done configuring firewall."